﻿//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.ComponentModel.Design.Data;
using System.Data;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.Design;


namespace MindFusion.Reporting.WinForms.Samples.CS.ReportEditor
{
	/// <summary>
	/// Represents the main application window.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			// Assoicate toolbar buttons with commands
			RegisterCommandListeners();

			categoriesTableAdapter.Fill(nwindDataSet.Categories);
			productsTableAdapter.Fill(nwindDataSet.Products);
			employeesTableAdapter.Fill(nwindDataSet.Employees);
			sales_by_CategoryTableAdapter.Fill(nwindDataSet.Sales_by_Category);

			// Add sample data sources
			MovieData data = new MovieData();
			reportEditor.AddDataSource(data.Items, "movies");
			reportEditor.AddDataSource(nwindDataSet, "nwindDataSet");

			UpdateTitle();
		}

		/// <summary>
		/// Form.OnClosing override.
		/// </summary>
		protected override void OnClosing(CancelEventArgs e)
		{
			e.Cancel = !ConfirmCloseReport();

			base.OnClosing(e);
		}

		/// <summary>
		/// Form.ProcessCmdKey override.
		/// </summary>
		protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
		{
			if (msg.Msg == 0x100 || // WM_KEYDOWN
				msg.Msg == 0x104) // WM_SYSKEYDOWN
			{
				// Check the previous key state to ensure that the command will be invoked only
				// once regardless of whether the user holds down the shortcut key
				if ((msg.LParam.ToInt64() & 0x40000000) == 0)
				{
					// Del = Delete Selection (Note: We don't use menu shortcut key to avoid
					// blocking all input of the DEL key)
					if ((keyData & Keys.KeyCode) == Keys.Delete)
					{
						if (reportEditor.IsDesignerFocused)
							return reportEditor.InvokeCommand(StandardCommands.Delete);
					}

					// Ctrl+PageDown = Change Tabs
					if ((keyData & Keys.Modifiers) == Keys.Control &&
						(keyData & Keys.KeyCode) == Keys.PageDown)
					{
						if (reportEditor.ActiveTabPage == ReportEditorTabPage.Design)
							reportEditor.ActiveTabPage = ReportEditorTabPage.Preview;
						else if (reportEditor.ActiveTabPage == ReportEditorTabPage.Preview)
							reportEditor.ActiveTabPage = ReportEditorTabPage.Design;
						return true;
					}

					// Ctrl+PageUp = Change Tabs
					if ((keyData & Keys.Modifiers) == Keys.Control &&
						(keyData & Keys.KeyCode) == Keys.PageUp)
					{
						if (reportEditor.ActiveTabPage == ReportEditorTabPage.Preview)
							reportEditor.ActiveTabPage = ReportEditorTabPage.Design;
						else if (reportEditor.ActiveTabPage == ReportEditorTabPage.Design)
							reportEditor.ActiveTabPage = ReportEditorTabPage.Preview;
						return true;
					}
				}
			}

			return base.ProcessCmdKey(ref msg, keyData);
		}

		/// <summary>
		/// Associates the toolbar buttons with their corresponding commands.
		/// </summary>
		private void RegisterCommandListeners()
		{
			RegisterCommandListener(StandardCommands.AlignLeft, alignLeftsToolButton);
			RegisterCommandListener(StandardCommands.AlignVerticalCenters, alignCentersToolButton);
			RegisterCommandListener(StandardCommands.AlignRight, alignRightsToolButton);
			RegisterCommandListener(StandardCommands.AlignTop, alignTopsToolButton);
			RegisterCommandListener(StandardCommands.AlignHorizontalCenters, alignMiddlesToolButton);
			RegisterCommandListener(StandardCommands.AlignBottom, alignBottomsToolButton);
			RegisterCommandListener(StandardCommands.SizeToControlWidth, makeSameWidthToolButton);
			RegisterCommandListener(StandardCommands.SizeToControlHeight, makeSameHeightToolButton);
			RegisterCommandListener(StandardCommands.SizeToControl, makeSameSizeToolButton);
			RegisterCommandListener(StandardCommands.SizeToGrid, sizeToGridToolButton);
			RegisterCommandListener(StandardCommands.HorizSpaceMakeEqual, makeHSpacingEqualToolButton);
			RegisterCommandListener(StandardCommands.HorizSpaceIncrease, increaseHSpacingToolButton);
			RegisterCommandListener(StandardCommands.HorizSpaceDecrease, decreaseHSpacingToolButton);
			RegisterCommandListener(StandardCommands.HorizSpaceConcatenate, removeHSpacingToolButton);
			RegisterCommandListener(StandardCommands.VertSpaceMakeEqual, makeVSpacingEqualToolButton);
			RegisterCommandListener(StandardCommands.VertSpaceIncrease, increaseVSpacingToolButton);
			RegisterCommandListener(StandardCommands.VertSpaceDecrease, decreaseVSpacingToolButton);
			RegisterCommandListener(StandardCommands.VertSpaceConcatenate, removeVSpacingToolButton);
			RegisterCommandListener(StandardCommands.CenterHorizontally, centerHorizontallyToolButton);
			RegisterCommandListener(StandardCommands.CenterVertically, centerVerticallyToolButton);
			RegisterCommandListener(StandardCommands.BringToFront, bringToFrontToolButton);
			RegisterCommandListener(StandardCommands.SendToBack, sendToBackToolButton);

			RegisterCommandListener(StandardCommands.Undo, undoMenuItem);
			RegisterCommandListener(StandardCommands.Redo, redoMenuItem);
			RegisterCommandListener(StandardCommands.Cut, cutMenuItem);
			RegisterCommandListener(StandardCommands.Copy, copyMenuItem);
			RegisterCommandListener(StandardCommands.Paste, pasteMenuItem);
			RegisterCommandListener(StandardCommands.Delete, deleteMenuItem);
			RegisterCommandListener(StandardCommands.SelectAll, selectAllMenuItem);
		}

		/// <summary>
		/// A helper method to register command listeners.
		/// </summary>
		private void RegisterCommandListener(CommandID id, ToolStripItem item)
		{
			item.Tag = id;
			reportEditor.RegisterCommandListener(id, new CommandListener(item));
		}

		/// <summary>
		/// Handles the ItemClicked event of the layout toolbar.
		/// </summary>
		private void OnToolItemClicked(object sender, ToolStripItemClickedEventArgs e)
		{
			if (e.ClickedItem == null)
				return;

			var id = e.ClickedItem.Tag as CommandID;
			if (id != null)
				reportEditor.InvokeCommand(id);
		}

		/// <summary>
		/// If the current report (if any) is dirty, prompts the user to save it.
		/// </summary>
		private bool ConfirmCloseReport()
		{
			if (reportEditor.Report == null)
				return true;

			if (reportEditor.IsReportDirty)
			{
				DialogResult result = MessageBox.Show(this, "Save changes before exiting?",
					"MindFusion.Reporting Editor", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);

				if (result == DialogResult.Cancel)
					return false;
				if (result == DialogResult.Yes)
				{
					if (!SaveReport(reportFileName, reportFileName))
						return false;
				}
			}

			return true;
		}

		/// <summary>
		/// Opens a new report. If the current report is dirty, the user
		/// is prompted to save it.
		/// </summary>
		private bool NewReport()
		{
			if (!ConfirmCloseReport())
				return false;

			reportEditor.SuspendPreviewUpdate();
			reportEditor.NewReport();
			reportEditor.ResumePreviewUpdate();

			reportFileName = null;
			UpdateTitle();

			return true;
		}

		/// <summary>
		/// Closes the current report. If the report is dirty, the user
		/// is prompted to save it.
		/// </summary>
		private bool CloseReport()
		{
			if (!ConfirmCloseReport())
				return false;

			reportEditor.CloseReport();
			reportFileName = null;
			UpdateTitle();

			return true;
		}

		/// <summary>
		/// Saves a report under the specified file. If the file name is
		/// null or empty the user is prompted for a file name through a dialog.
		/// </summary>
		private bool SaveReport(string fileName, string originalFileName)
		{
			if (string.IsNullOrEmpty(fileName))
			{
				SaveFileDialog dialog = new SaveFileDialog();
				dialog.Filter = "XML Files (*.xml)|*.xml|All Files|*.*||";
				dialog.FileName = string.IsNullOrEmpty(originalFileName) ? "Untitled.xml" : originalFileName;
				if (dialog.ShowDialog(this) != DialogResult.OK)
					return false;

				reportFileName = dialog.FileName;
			}
			else
			{
				reportFileName = fileName;
			}

			reportEditor.SaveReport(reportFileName);
			UpdateTitle();

			return true;
		}

		/// <summary>
		/// Opens a report from a file. The file is acquired through a dialog.
		/// The user is prompted to save the currently open report (if any and dirty).
		/// </summary>
		private bool OpenReport()
		{
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.Filter = "XML Files (*.xml)|*.xml|All Files|*.*||";
			if (dialog.ShowDialog(this) != DialogResult.OK)
				return false;

			if (!ConfirmCloseReport())
				return false;

			reportFileName = dialog.FileName;
			reportEditor.SuspendPreviewUpdate();
			reportEditor.OpenReport(reportFileName);
			reportEditor.ResumePreviewUpdate();
			UpdateTitle();

			return true;
		}

		/// <summary>
		/// Handles the Click event of the New menu item.
		/// </summary>
		private void OnNewMenuItemClick(object sender, EventArgs e)
		{
			NewReport();
		}

		/// <summary>
		/// Handles the Click event of the Open menu item.
		/// </summary>
		private void OnOpenMenuItemClick(object sender, EventArgs e)
		{
			OpenReport();
		}

		/// <summary>
		/// Handles the Click event of the Save menu item.
		/// </summary>
		private void OnSaveMenuItemClick(object sender, EventArgs e)
		{
			SaveReport(reportFileName, reportFileName);
		}

		/// <summary>
		/// Handles the Click event of the Save As menu item.
		/// </summary>
		private void OnSaveAsMenuItemClick(object sender, EventArgs e)
		{
			SaveReport(null, reportFileName);
		}

		/// <summary>
		/// Handles the Click event of the Close menu item.
		/// </summary>
		private void OnCloseMenuItemClick(object sender, EventArgs e)
		{
			CloseReport();
		}

		/// <summary>
		/// Handles the Click event of the Preview menu item.
		/// </summary>
		private void OnPrintPreviewMenuItemClick(object sender, EventArgs e)
		{
			// Make sure the report is run
			reportEditor.RunReport();

			var printPreviewForm = new PrintPreviewForm();
			printPreviewForm.Report = reportEditor.Report;
			printPreviewForm.ShowDialog(this);
		}

		/// <summary>
		/// Handles the Click event of the Print menu item.
		/// </summary>
		private void OnPrintMenuItemClick(object sender, EventArgs e)
		{
			var printer = new ReportPrinter();
			printer.Report = reportEditor.Report;
			printer.Report.Run();
			printer.Print();
		}

		/// <summary>
		/// Handles the Click event of the Exit menu item.
		/// </summary>
		private void OnExitMenuItemClick(object sender, EventArgs e)
		{
			this.Close();
		}

		/// <summary>
		/// Handles the Click event of the Undo menu item.
		/// </summary>
		private void OnUndoMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.Undo();
		}

		/// <summary>
		/// Handles the Click event of the Redo menu item.
		/// </summary>
		private void OnRedoMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.Redo();
		}

		/// <summary>
		/// Handles the Click event of the Cut menu item.
		/// </summary>
		private void OnCutMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.InvokeCommand(StandardCommands.Cut);
		}

		/// <summary>
		/// Handles the Click event of the Copy menu item.
		/// </summary>
		private void OnCopyMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.InvokeCommand(StandardCommands.Copy);
		}

		/// <summary>
		/// Handles the Click event of the Paste menu item.
		/// </summary>
		private void OnPasteMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.InvokeCommand(StandardCommands.Paste);
		}

		/// <summary>
		/// Handles the Click event of the Delete menu item.
		/// </summary>
		private void OnDeleteMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.InvokeCommand(StandardCommands.Delete);
		}

		/// <summary>
		/// Handles the Click event of the Select All menu item.
		/// </summary>
		private void OnSelectAllMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.InvokeCommand(StandardCommands.SelectAll);
		}

		/// <summary>
		/// Handles the Click event of the Properties menu.
		/// </summary>
		private void OnPropertiesMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.InvokeCommand(StandardCommands.PropertiesWindow);
		}

		/// <summary>
		/// Handles the Click event of the Toolbox menu.
		/// </summary>
		private void OnToolboxMenuItemClick(object sender, EventArgs e)
		{
			reportEditor.InvokeCommand(MenuCommands.ToolboxWindow);
		}

		/// <summary>
		/// Handles the Click event of the Toolbox menu.
		/// </summary>
		private void OnAboutMenuItemClick(object sender, EventArgs e)
		{
			new AboutForm().ShowDialog(this);
		}

		/// <summary>
		/// Handles the PreviewUpdating event of the report editor.
		/// </summary>
		private void OnReportEditorPreviewUpdating(object sender, EventArgs e)
		{
			statusLabel.Text = "Updating Preview...";
			Application.DoEvents();
		}

		/// <summary>
		/// Handles the PreviewUpdated event of the report editor.
		/// </summary>
		private void OnReportEditorPreviewUpdated(object sender, EventArgs e)
		{
			statusLabel.Text = "";
			Application.DoEvents();
		}

		/// <summary>
		/// Handles the ReportDirtyChanged event of the report editor.
		/// </summary>
		private void OnReportEditorDirtyChanged(object sender, EventArgs e)
		{
			UpdateTitle();
		}

		/// <summary>
		/// Updates the window's title.
		/// </summary>
		private void UpdateTitle()
		{
			if (reportEditor.Report == null)
			{
				Text = "MindFusion.Reporting Editor";
			}
			else
			{
				string fileName = "Untitled.xml";
				if (!string.IsNullOrEmpty(reportFileName))
					fileName = Path.GetFileName(reportFileName);
				string modificationMark = reportEditor.IsReportDirty ? "*" : "";

				Text = string.Format("{0}{1} - MindFusion.Reporting Editor", fileName, modificationMark);
			}
		}


		/// <summary>
		/// An implementation of the ICommandListener interface for toolstrip items.
		/// </summary>
		private class CommandListener : ICommandListener
		{
			/// <summary>
			/// Initializes a new instance of the CommandListener class.
			/// </summary>
			public CommandListener(ToolStripItem item)
			{
				this.item = item;
			}

			/// <summary>
			/// ICommandListener&lt;ToolStripItem&gt;.UpdateState implementation.
			/// </summary>
			public void UpdateState(bool enabled, bool visible)
			{
				item.Enabled = enabled;
				item.Visible = visible;
			}


			private ToolStripItem item;
		}


		private string reportFileName;
	}
}
